<?php


use \Illuminate\Database\Capsule\Manager as Capsule;

function joker_validateLatin($string)
{

    return preg_match("/^[\w\d\s.,-]*$/", $string);

}

function joker_GetNameservers($params)
{


    $joker_config = joker_getConfig();
    $c = joker_getConnection();

    $sessId = joker_getSessionId($joker_config);
    $regParams = [
        'domain' => $params['original']['domainname'],
    ];
    $c->execute_request('query-whois', $regParams, $response, $sessId);
    if ($response['response_header']['status-code']!=0)
        return [ 'error' => 'error' ];
    $nslist = [ ];

    $body = @explode("\n", $response['response_body']);

    foreach ($body as $row) {

        $row = explode(':', $row);

        if ($row[0]==='domain.nservers.nserver.handle')
            $nslist[] = $row[1];
    }

    foreach ($nslist as $index => $ns)
        $values[ 'ns' . ($index + 1) ] = $ns;


}

function joker_SaveNameservers($params)
{

    $joker_config = joker_getConfig();


    $c = joker_getConnection();


    $sessId = joker_getSessionId($joker_config);


    $nameserver1 = trim(strtolower($params["ns1"]));
    $nameserver2 = trim(strtolower($params["ns2"]));
    $nameserver3 = trim(strtolower($params["ns3"]));
    $nameserver4 = trim(strtolower($params["ns4"]));
    $nameserver5 = trim(strtolower($params["ns5"]));


    $nslist = $nameserver1;

    if ($nameserver2)
        $nslist .= ':' . $nameserver2;

    if ($nameserver3)
        $nslist .= ':' . $nameserver3;
    if ($nameserver4)
        $nslist .= ':' . $nameserver4;
    if ($nameserver5)
        $nslist .= ':' . $nameserver5;


    $regParams = [
        'domain'  => $params['original']['domainname'],
        'ns-list' => $nslist,


    ];

    joker_log('domain-modify before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

    $c->execute_request('domain-modify', $regParams, $response, $sessId);

    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('domain-modify after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);

    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }


}

function joker_GetRegistrarLock($params)
{
    $joker_config = joker_getConfig();
    $c = joker_getConnection();

    $sessId = joker_getSessionId($joker_config);
    $regParams = [
        'domain' => $params['original']['domainname'],
    ];
    $c->execute_request('query-whois', $regParams, $response, $sessId);

    $body = @explode("\n", $response['response_body']);

    $lockstatus = null;
    foreach ($body as $row) {

        $row = explode(':', $row);

        if ($row[0]==='domain.status')
            $lockstatus = trim($row[1]);
    }


    if ($lockstatus==='production')
        $lockstatus = 'unlocked';
    else
        $lockstatus = 'locked';


    return $lockstatus;
}

function joker_SaveRegistrarLock($params)
{
    $lockstatus = $params["lockenabled"];


    $joker_config = joker_getConfig();
    $c = joker_getConnection();

    $sessId = joker_getSessionId($joker_config);
    $regParams = [
        'domain' => $params['original']['domainname'],
    ];
    if ($lockstatus==='locked') {
        joker_log('domain-lock before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

        $c->execute_request('domain-lock', $regParams, $response, $sessId);
        $resStatus = 'new';
        if ($response['response_header']['status-code']!=0) {
            $resStatus = 'failed';
        }
        joker_log('domain-lock after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
        if ($response['response_header']['status-code']!=0) {
            return [ 'error' => json_encode($response['response_header']['error']) ];
        }
    } else {
        joker_log('domain-unlock before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

        $c->execute_request('domain-unlock', $regParams, $response, $sessId);
        $resStatus = 'new';
        if ($response['response_header']['status-code']!=0) {
            $resStatus = 'failed';
        }
        joker_log('domain-unlock after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
        if ($response['response_header']['status-code']!=0) {
            return [ 'error' => json_encode($response['response_header']['error']) ];
        }
    }


    $values = null;

    return $values;
}


function joker_RegisterDomain($params)
{


    $joker_config = joker_getConfig();

    if (!isset($joker_config['contacts'][ $params['tld'] ]))
        return [ 'error' => 'invalid tld' ];


    $c = joker_getConnection();

    $tld = $params['tld'];

    $sessId = joker_getSessionId($joker_config);


    $nameserver1 = strtolower($params["ns1"]);
    $nameserver2 = strtolower($params["ns2"]);
    $nameserver3 = strtolower($params["ns3"]);
    $nameserver4 = strtolower($params["ns4"]);
    $nslist = $nameserver1;


    if ($nameserver2)
        $nslist .= ':' . $nameserver2;
    if ($nameserver3)
        $nslist .= ':' . $nameserver3;
    if ($nameserver4)
        $nslist .= ':' . $nameserver4;


    $regperiod = $params["regperiod"];
    $regParams = [
        'domain'    => $params['original']['domainname'],
        'ns-list'   => $nslist,
        'period'    => $regperiod * 12,
        'owner-c'   => $joker_config['contacts'][ $tld ],
        'admin-c'   => $joker_config['contacts'][ $tld ],
        'billing-c' => $joker_config['contacts'][ $tld ],
        'tech-c'    => $joker_config['contacts'][ $tld ],

    ];
    joker_log('domain-register before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

    $c->execute_request('domain-register', $regParams, $response, $sessId);
    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('domain-register after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }
}

function joker_TransferDomain($params)
{
    $username = $params["Username"];
    $password = $params["Password"];
    $testmode = $params["TestMode"];
    $tld = $params["tld"];
    $sld = $params["sld"];
    $regperiod = $params["regperiod"];
    $transfersecret = $params["transfersecret"];
    $nameserver1 = $params["ns1"];
    $nameserver2 = $params["ns2"];
    # Registrant Details
    $RegistrantFirstName = $params["firstname"];
    $RegistrantLastName = $params["lastname"];
    $RegistrantAddress1 = $params["address1"];
    $RegistrantAddress2 = $params["address2"];
    $RegistrantCity = $params["city"];
    $RegistrantStateProvince = $params["state"];
    $RegistrantPostalCode = $params["postcode"];
    $RegistrantCountry = $params["country"];
    $RegistrantEmailAddress = $params["email"];
    $RegistrantPhone = $params["phonenumber"];
    # Admin Details
    $AdminFirstName = $params["adminfirstname"];
    $AdminLastName = $params["adminlastname"];
    $AdminAddress1 = $params["adminaddress1"];
    $AdminAddress2 = $params["adminaddress2"];
    $AdminCity = $params["admincity"];
    $AdminStateProvince = $params["adminstate"];
    $AdminPostalCode = $params["adminpostcode"];
    $AdminCountry = $params["admincountry"];
    $AdminEmailAddress = $params["adminemail"];
    $AdminPhone = $params["adminphonenumber"];
    # Put your code to transfer domain here


    $joker_config = joker_getConfig();

    if (!isset($joker_config['contacts'][ $params['tld'] ]))
        return [ 'error' => 'invalid tld' ];


    $nameserver1 = strtolower($params["ns1"]);
    $nameserver2 = strtolower($params["ns2"]);
    $nameserver3 = strtolower($params["ns3"]);
    $nameserver4 = strtolower($params["ns4"]);
    $nslist = $nameserver1;

    if ($nameserver2)
        $nslist .= ':' . $nameserver2;
    if ($nameserver3)
        $nslist .= ':' . $nameserver3;
    if ($nameserver4)
        $nslist .= ':' . $nameserver4;

    $c = joker_getConnection();

    $tld = $params['tld'];

    $sessId = joker_getSessionId($joker_config);


    $regperiod = $params["regperiod"];
    $regParams = [
        'domain'           => $params['original']['domainname'],
        'transfer-auth-id' => $transfersecret,
        'owner-c'          => $joker_config['contacts'][ $tld ],
        'admin-c'          => $joker_config['contacts'][ $tld ],
        'billing-c'        => $joker_config['contacts'][ $tld ],
        'tech-c'           => $joker_config['contacts'][ $tld ],
        'ns-list'          => $nslist,

    ];
    joker_log('domain-transfer-in-reseller before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

    $c->execute_request('domain-transfer-in-reseller', $regParams, $response, $sessId);
    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('domain-transfer-in-reseller after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }

}

function joker_RenewDomain($params)
{
    $joker_config = joker_getConfig();
    $c = joker_getConnection();
    $sessId = joker_getSessionId($joker_config);
    $regperiod = $params["regperiod"];
    $regParams = [
        'domain' => $params['original']['domainname'],
        'period' => $regperiod * 12,
    ];
    joker_log('domain-renew before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

    $c->execute_request('domain-renew', $regParams, $response, $sessId);
    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('domain-renew after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }


}

function joker_GetContactDetails($params)
{
    $joker_config = joker_getConfig();

    $sessId = joker_getSessionId($joker_config);
    $whoisData = joker_getWhois($params['original']['domainname'], $sessId);
    foreach ($whoisData as $row) {
        if ($row[0]=='domain.name') {
            $name = explode(' ', $row[1]);
            $values["Registrant"]["First Name"] = $name[0];
            $values["Registrant"]["Last Name"] = $name[1];

        }
        if ($row[0]=='domain.email') {

            $values["Registrant"]["Email"] = $row[1];


        }
        if ($row[0]=='domain.organization') {

            $values["Registrant"]["Organization"] = $row[1];


        }
        if ($row[0]=='domain.address-1') {

            $values["Registrant"]["Address-1"] = $row[1];


        }
        if ($row[0]=='domain.postal-code') {

            $values["Registrant"]["Postal Code"] = $row[1];


        }
        if ($row[0]=='domain.phone') {

            $values["Registrant"]["Phone"] = $row[1];


        }
        if ($row[0]=='domain.city') {

            $values["Registrant"]["City"] = $row[1];


        }
        if ($row[0]=='domain.country') {

            $values["Registrant"]["Country Code"] = $row[1];

        }

    }

    return $values;
}

function joker_SaveContactDetails($params)
{
    $username = $params["Username"];
    $password = $params["Password"];
    $testmode = $params["TestMode"];

    $tld = $params["tld"];
    $sld = $params["sld"];
    # Data is returned as specified in the GetContactDetails() function
    $firstname = $params["contactdetails"]["Registrant"]["First Name"];


    $lastname = $params["contactdetails"]["Registrant"]["Last Name"];
    $email = $params["contactdetails"]["Registrant"]["Email"];
    $organization = $params["contactdetails"]["Registrant"]["Organization"];
    $city = $params["contactdetails"]["Registrant"]["City"];
    $phone = $params["contactdetails"]["Registrant"]["Phone"];
    $address = $params["contactdetails"]["Registrant"]["Address-1"];
    $postalCode = $params["contactdetails"]["Registrant"]["Postal Code"];
    $country = $params["contactdetails"]["Registrant"]["Country Code"];

    if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
        return [ 'error' => 'email not valid' ];
    }
    if (!joker_validateLatin($firstname) ||
        !joker_validateLatin($lastname) ||
        !joker_validateLatin($organization) ||
        !joker_validateLatin($city) ||
        !joker_validateLatin($address) ||
        !joker_validateLatin($country)
    ) {
        return [ 'error' => 'only english' ];
    }



    $joker_config = joker_getConfig();
    $c = joker_getConnection();

    $sessId = joker_getSessionId($joker_config);
    $regParams = [
        'domain'       => $params['original']['domainname'],
        'fname'        => $firstname,
        'lname'        => $lastname,
        'email'        => $email,
        'organization' => $organization,
        'city'         => $city,
        'phone'        => $phone,
        'address-1'    => $address,
        'postal-code'  => $postalCode,
        'country'      => $country,
    ];

    joker_log('domain-owner-change before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

    $c->execute_request('domain-owner-change', $regParams, $response, $sessId);
    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('domain-owner-change after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }

    return null;
}

function joker_GetEPPCode($params)
{
    $joker_config = joker_getConfig();
    $c = joker_getConnection();

    $sessId = joker_getSessionId($joker_config);
    $regParams = [
        'domain' => $params['original']['domainname'],
    ];

    joker_log('domain-transfer-get-auth-id before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

    $c->execute_request('domain-transfer-get-auth-id', $regParams, $response, $sessId);
    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('domain-transfer-get-auth-id after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);

    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }

    $whoisdata = joker_getWhois($params['original']['domainname'], $sessId);
    $email = null;

    foreach ($whoisdata as $row) {
        if ($row[0]==='domain.email') {
            $email = $row[1];
            break;
        }
    }

    $values["eppcode"] = "email to: $email";

    return $values;

}

function joker_RegisterNameserver($params)
{

    $nameserver = $params["nameserver"];
    $ipaddress = $params["ipaddress"];

    $joker_config = joker_getConfig();
    $c = joker_getConnection();


    $sessId = joker_getSessionId($joker_config);
    $regParams = [
        'host' => $nameserver,
        'ip'   => $ipaddress,
    ];

    $res = filter_var($ipaddress, FILTER_VALIDATE_IP);
    if (!$res)
        return [ 'error' => 'invalid-ip' ];

    $c->execute_request('query-ns-list', [ 'pattern' => $nameserver ], $response, $sessId);

    if (trim($response['response_body'])===$nameserver)
        return [ "error" => 'nameserver-exist' ];
    joker_log('ns-create before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');
    $c->execute_request('ns-create', $regParams, $response, $sessId);
    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('ns-create after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }

}

function joker_ModifyNameserver($params)
{

    $nameserver = $params["nameserver"];
    $currentipaddress = $params["currentipaddress"];
    $newipaddress = $params["newipaddress"];


    $joker_config = joker_getConfig();
    $c = joker_getConnection();


    $sessId = joker_getSessionId($joker_config);
    $regParams = [
        'host' => $nameserver,
        'ip'   => $newipaddress,
    ];

    $res = filter_var($newipaddress, FILTER_VALIDATE_IP);
    if (!$res)
        return [ 'error' => 'invalid-ip' ];

    $c->execute_request('query-ns-list', [ 'pattern' => $nameserver ], $response, $sessId);

    if (trim($response['response_body'])=='')
        return [ "error" => 'nameserver-not-found' ];


    joker_log('ns-modify before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $regParams, $params['original']['domainname'], 'success');

    $c->execute_request('ns-modify', $regParams, $response, $sessId);
    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('ns-modify after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }

}

function joker_DeleteNameserver($params)
{

    $nameserver = $params["nameserver"];

    $joker_config = joker_getConfig();
    $c = joker_getConnection();


    $sessId = joker_getSessionId($joker_config);
    $delParams = [ 'host' => $nameserver ];
    joker_log('ns-delete before', $_SESSION['adminid'], $_SESSION['uid'], null, null, $delParams, $params['original']['domainname'], 'success');

    $c->execute_request('ns-delete', $delParams, $response, $sessId);
    $resStatus = 'new';
    if ($response['response_header']['status-code']!=0) {
        $resStatus = 'failed';
    }
    joker_log('ns-delete after', $_SESSION['adminid'], $_SESSION['uid'], $response['response_header']['proc-id'], $response['response_header']['tracking-id'], $response['raw'], $params['original']['domainname'], $resStatus);
    if ($response['response_header']['status-code']!=0) {
        return [ 'error' => json_encode($response['response_header']['error']) ];
    }

}


function joker_getSessionId($params)
{

    $c = joker_getConnection();

    $username = $params["username"];
    $password = $params["password"];


    $sessId = '';
    $response = '';

    $c->execute_request('login', [ 'username' => $username, 'password' => $password ], $response, $sessId);

    if ($response['response_header']['status-code']!=0) {
        return false;
    }

    $sessId = $response['response_header']['auth-sid'];

    return $sessId;
}

function joker_getConnection()
{
    require_once __DIR__ . '/lib/cls_connect.php';

    return new Connect(joker_getConfig());

}

function joker_getConfig()
{
    global $joker_config;

    if ($joker_config)
        return $joker_config;

    $config = require __DIR__ . '/config.php';

    $joker_config = $config;

    return $joker_config;
}

function joker_getWhois($domain, $sessId)
{
    $joker_config = joker_getConfig();
    $c = joker_getConnection();

    $regParams = [
        'domain' => $domain,
    ];
    $c->execute_request('query-whois', $regParams, $response, $sessId);


    $nslist = [ ];

    $body = @explode("\n", $response['response_body']);

    $lockstatus = null;
    $whoisData = [ ];
    foreach ($body as $row) {

        $row = explode(':', $row);
        $whoisData[] = [ trim($row[0]), trim($row[1]) ];


    }

    return $whoisData;
}

function joker_log($request, $adminId, $userId, $procId, $trackId, $data, $domain, $status = 'new')
{

    if (is_array($data)
    ) {
        $data = json_encode($data);

    }
    Capsule::table('mod_joker_trxlog')->insert([
        'userid'     => $userId,
        'adminid'    => $adminId,
        'domain'     => $domain,
        'trackid'    => $trackId,
        'procid'     => $procId,
        'data'       => $data,
        'request'    => $request,
        'status'     => $status,
        'created_at' => date('Y-m-d H:i:s'),
    ]);
    
}

?>